# Processing output files

File containing general information about puzzles, its location within initial alignment, and solutions. Partially contains "pikdik" information for a quick lookup.

Location: https://dataset.cs.mcgill.ca/bls/data_release/phase1/processing_output/

## Files:

- extract_output_1_2021-06-19.pickle
- extract_output_2_2021-06-19.pickle
- extract_output_3_2021-06-19.pickle
- extract_output_4_2021-06-19.pickle

## Prerequisites

All the pickle files described in this section use
[dnapuzzle.py](https://dataset.cs.mcgill.ca/bls/data_release/phase1/supplementary/dnapuzzle.py).
Make sure you have ``dnapuzzle.py`` in the working directory.

[Pandas](https://pandas.pydata.org/getting_started.html) library is used in th Code section,
however it is not required to open the file.

## Code

To open the pickle file (replace filepath with appropriate string):

```python
import pandas as pd
from dnapuzzle import Puzzle
filepath = "extract_output_1_2021-06-19.pickle"
obj = pd.read_pickle(filepath)
```

## Format 

Pickle file encodes a dictionary where:
- keys : stringified list of column indexes in original alignment (e.g '[99, 100, 101, 102, 103, 104, 105]')
- values : list of dictionaries that describe distinct puzzles (see below)

### Puzzle description dictionary

Pickle values list element contains the following attributes:

- originalCode: [str] puzzle ID in BLS system (e.g. "VeoWZMUzLmzgbU3e"),
- pikdik: [dnapuzzle.Puzzle] Puzzle Object (see below),
- nGaps: [int] number of gaps allowed in a puzzle (e.g. 6),
- score: [float] expected score for a particular puzzle (e.g. 17.0),
- pareto: [str] type of strategy used (e.g. "FalseSubopt"),
- playerSolutions: [list] list of players' solutions (e.g. [['TTG-A', 'GT--AA', 'GCGA', '-TGCA', 'CT', 'CT-GA']]),
- playerIDs: [list] list of player IDs that submitted solutions described in playerSolutions (e.g. ['1269979'])

### pikdik: Puzzle Object

- pikdik.puzzle: [list] collapsed symbol-wise split puzzle sequences (e.g. [['T', 'T', 'G', 'A', '-', '-', '-'], ['G', 'T', 'A', 'A', '-', '-', '-'], ['G', 'C', 'G', 'A', '-', '-', '-'], ['T', 'G', 'C', 'A', '-', '-', '-'], ['C', 'T', '-', '-', '-', '-', '-'], ['C', 'T', 'G', 'A', '-', '-', '-']] )
- pikdik.par_puzzle: [list] symbol-wise split pareto solution of the puzzle (e.g. [['T', 'T', '-', 'G', 'A', '-', '-'], ['G', 'T', '-', 'A', 'A', '-', '-'], ['G', 'C', '-', 'G', 'A', '-', '-'], ['T', 'G', '-', 'C', 'A', '-', '-'], ['C', 'T', '-', '-', '-', '-', '-'], ['C', 'T', '-', 'G', 'A', '-', '-']] )
- pikdik.columns: [str(list)] column indexes in original alignment (e.g. '[99, 100, 101, 102, 103, 104, 105]' )
- pikdik.flanks: [list] column indexes used as a flanks for a puzzle (e.g [105, 106])
- pikdik.consensus: [list] predefined guides for a puzzle (used in scoring) (e.g. [('C', 'T'), ('C', 'T'), ('-', 'C'), ('C', 'T'), ('-', 'A'), ('G', 'A'), ('-', 'C'), ('-', 'G')] )
- pikdik.cons_scores: [list] counter for each nucleotide from consenus to appear in the alignment (e.g. [(6287, 2552), (3416, 2688), (9666, 1), (2938, 2669), (9545, 55)] )
- pikdik.n: [int] number of sequences (e.g. 6)
- pikdik.bonus: [float] multiplier for aligned "line" perfectly matching given consensus / guide (e.g. 1.15)
- pikdik.level: [int] difficulty level of the puzzle in range [1, 9] (e.g. 2)

